using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Runtime.Serialization;
using System.Globalization;
using System.ComponentModel.Design.Serialization;
using System.Reflection;

namespace PropertyPersistedByStaticInstanceSample {
	public class PPBSISControl : Control {
		SerializableClass sp, nsp;
		public PPBSISControl() {
			sp = SerializableClass.StaticProperty;
			nsp = new SerializableClass(2);
		}
		public SerializableClass StaticProperty {
			get {
				return sp;
			} set {
				sp = value;
			}
		}
		public SerializableClass NonStaticProperty {
			get {
				return nsp;
			} set {
				nsp = value;
			}
		}
	}
	[Serializable]
	[TypeConverter(typeof(SerializableClass._SerializableClassConverter))]
	public class SerializableClass : ISerializable {
		internal class _SerializableClassConverter : ExpandableObjectConverter {
			public override bool CanConvertTo(ITypeDescriptorContext context, Type dst_type) {
				if (dst_type == typeof(InstanceDescriptor))
					return true;
				return base.CanConvertTo(context, dst_type);
			}
			public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type dst_type) {  
				if (dst_type == typeof(InstanceDescriptor)) {
					SerializableClass sd = value as SerializableClass;
					if (sd == SerializableClass.StaticProperty) {
						PropertyInfo pi = typeof(SerializableClass).GetProperty("StaticProperty");
						return new InstanceDescriptor(pi, null);
					} else {
						ConstructorInfo ci = typeof(SerializableClass).GetConstructor(new Type[]{typeof(int)});
						return new InstanceDescriptor(ci, new object[]{sd.Data});
					}
				}
				return base.ConvertTo(context, culture, value, dst_type);
			}
		}
		[Serializable]
		class StaticFieldSerializationHelper : IObjectReference {
			object IObjectReference.GetRealObject(StreamingContext context) {
				Console.WriteLine(context);
				return sf;
			}
		}
		static readonly SerializableClass sf = new SerializableClass(1);
		int data;
		public SerializableClass(int data) {
			Data = data;
		}
		SerializableClass(SerializationInfo info, StreamingContext context) {
			data = (int)info.GetValue("data", typeof(int));
		}
		void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context) {
			if (this == sf)
				info.SetType(typeof(StaticFieldSerializationHelper));
			else
				info.AddValue("data", data);
		}
		public int Data {
			get {
				return data;
			}
			set {
				data = value;
			}
		}
		public static SerializableClass StaticProperty {
			get {
				return sf;
			}
		}
	}
}
